var __loader = (function() {

var loader = {};

loader.packages = {};

loader.packagesLinenoOrder = [{ filename: 'loader.js', lineno: 0 }];

loader.fileExts = ['?', '?.js', '?.json'];
loader.folderExts = ['?/index.js', '?/index.json'];
loader.builtins = ['safe'];

loader.basepath = function(path) {
  return path.replace(/[^\/]*$/, '');
};

loader.joinpath = function() {
  var result = arguments[0];
  for (var i = 1; i < arguments.length; ++i) {
    if (arguments[i][0] === '/') {
      result = arguments[i];
    } else {
      result += '/' + arguments[i];
    }
  }

  if (result[0] === '/') {
    result = result.substr(1);
  }
  return result;
};

var replace = function(a, regexp, b) {
  var z;
  do {
    z = a;
  } while (z !== (a = a.replace(regexp, b)));
  return z;
};

loader.normalize = function(path) {
  path = replace(path, /(?:(^|\/)\.?\/)+/g, '$1');
  path = replace(path, /[^\/]*\/\.\.\//, '');
  path = path.replace(/\/\/+/g, '/');
  path = path.replace(/^\//, '');
  return path;
};

function _require(module) {
  if (module.exports) {
    return module.exports;
  }

  var require = function(path) { return loader.require(path, module); };

  module.exports = {};
  module.loader(module.exports, module, require);
  module.loaded = true;

  return module.exports;
}

loader.require = function(path, requirer) {
  var module = loader.getPackage(path, requirer);
  if (!module) {
    throw new Error("Cannot find module '" + path + "'");
  }

  return _require(module);
};

var compareLineno = function(a, b) { return a.lineno - b.lineno; };

loader.define = function(path, lineno, loadfun) {
  var module = {
    filename: path,
    lineno: lineno,
    loader: loadfun,
  };

  loader.packages[path] = module;
  loader.packagesLinenoOrder.push(module);
  loader.packagesLinenoOrder.sort(compareLineno);
};

loader.getPackageForPath = function(path) {
  return loader.getPackageForFile(path) || loader.getPackageForDirectory(path);
};

loader.getPackage = function(path, requirer) {
  var module;
  var fullPath;
  if (requirer && requirer.filename) {
    fullPath = loader.joinpath(loader.basepath(requirer.filename), path);
  } else {
    fullPath = path;
  }

  if (loader.builtins.indexOf(path) !== -1) {
    return loader.packages[path];
  }

  // Try loading the module from a path, if it is trying to load from a path.
  if (path.substr(0, 2) === './' || path.substr(0, 1) === '/' || path.substr(0, 3) === '../') {
    module = loader.getPackageForPath(fullPath);
  }

  if (!module) {
    module = loader.getPackageFromBuildOutput(path);
  }

  if (!module) {
    module = loader.getPackageForNodeModule(path);
  }

  return module;
};

loader.getPackageForFile = function(path) {
  path = loader.normalize(path);

  var module;
  var fileExts = loader.fileExts;
  for (var i = 0, ii = fileExts.length; !module && i < ii; ++i) {
    var filepath = fileExts[i].replace('?', path);
    module = loader.packages[filepath];
  }

  return module;
};

loader.getPackageForDirectory = function(path) {
  path = loader.normalize(path);

  var module;
  var packagePackage = loader.packages[loader.joinpath(path, 'package.json')];
  if (packagePackage) {
    var info = _require(packagePackage);
    if (info.main) {
      module = loader.getPackageForFile(loader.joinpath(path, info.main));
    }
  }

  if (!module) {
    module = loader.getPackageForFile(loader.joinpath(path, 'index'));
  }

  return module;
};

loader.getPackageFromBuildOutput = function(path) {
  var moduleBuildPath = loader.normalize(loader.joinpath('build', 'js', path));

  return loader.getPackageForPath(moduleBuildPath);
};

// Nested node_modules are banned, so we can do a simple search here.
loader.getPackageForNodeModule = function(path) {
  var modulePath = loader.normalize(loader.joinpath('node_modules', path));

  return loader.getPackageForPath(modulePath);
};

loader.getPackageByLineno = function(lineno) {
  var packages = loader.packagesLinenoOrder;
  var module;
  for (var i = 0, ii = packages.length; i < ii; ++i) {
    var next = packages[i];
    if (next.lineno > lineno) {
      break;
    }
    module = next;
  }
  return module;
};

return loader;

})();

__loader.define('safe', 181, function(exports, module, require) {
/* safe.js - Building a safer world for Pebble.JS Developers
 *
 * This library provides wrapper around all the asynchronous handlers that developers
 * have access to so that error messages are caught and displayed nicely in the pebble tool
 * console.
 */

/* global __loader */

var safe = {};

/* The name of the concatenated file to translate */
safe.translateName = 'pebble-js-app.js';

safe.indent = '    ';

/* Translates a source line position to the originating file */
safe.translatePos = function(name, lineno, colno) {
  if (name === safe.translateName) {
    var pkg = __loader.getPackageByLineno(lineno);
    if (pkg) {
      name = pkg.filename;
      lineno -= pkg.lineno;
    }
  }
  return name + ':' + lineno + ':' + colno;
};

var makeTranslateStack = function(stackLineRegExp, translateLine) {
  return function(stack, level) {
    var lines = stack.split('\n');
    var firstStackLine = -1;
    for (var i = lines.length - 1; i >= 0; --i) {
      var m = lines[i].match(stackLineRegExp);
      if (!m) {
        continue;
      }
      var line = lines[i] = translateLine.apply(this, m);
      if (line) {
        firstStackLine = i;
        if (line.indexOf(module.filename) !== -1) {
          lines.splice(i, 1);
        }
      } else {
        lines.splice(i, lines.length - i);
      }
    }
    if (firstStackLine > -1) {
      lines.splice(firstStackLine, level);
    }
    return lines;
  };
};

/* Translates a node style stack trace line */
var translateLineV8 = function(line, msg, scope, name, lineno, colno) {
  var pos = safe.translatePos(name, lineno, colno);
  return msg + (scope ? ' ' + scope + ' (' + pos + ')' : pos);
};

/* Matches <msg> (<scope> '(')? <name> ':' <lineno> ':' <colno> ')'? */
var stackLineRegExpV8 = /(.+?)(?:\s+([^\s]+)\s+\()?([^\s@:]+):(\d+):(\d+)\)?/;

safe.translateStackV8 = makeTranslateStack(stackLineRegExpV8, translateLineV8);

/* Translates an iOS stack trace line to node style */
var translateLineIOS = function(line, scope, name, lineno, colno) {
  var pos = safe.translatePos(name, lineno, colno);
  return safe.indent + 'at ' + (scope ? scope  + ' (' + pos + ')' : pos);
};

/* Matches (<scope> '@' )? <name> ':' <lineno> ':' <colno> */
var stackLineRegExpIOS = /(?:([^\s@]+)@)?([^\s@:]+):(\d+):(\d+)/;

safe.translateStackIOS = makeTranslateStack(stackLineRegExpIOS, translateLineIOS);

/* Translates an Android stack trace line to node style */
var translateLineAndroid = function(line, msg, scope, name, lineno, colno) {
  if (name !== 'jskit_startup.js') {
    return translateLineV8(line, msg, scope, name, lineno, colno);
  }
};

/* Matches <msg> <scope> '('? filepath <name> ':' <lineno> ':' <colno> ')'? */
var stackLineRegExpAndroid = /^(.*?)(?:\s+([^\s]+)\s+\()?[^\s\(]*?([^\/]*?):(\d+):(\d+)\)?/;

safe.translateStackAndroid = makeTranslateStack(stackLineRegExpAndroid, translateLineAndroid);

/* Translates a stack trace to the originating files */
safe.translateStack = function(stack, level) {
  level = level || 0;
  if (Pebble.platform === 'pypkjs') {
    return safe.translateStackV8(stack, level);
  } else if (stack.match('com.getpebble.android')) {
    return safe.translateStackAndroid(stack, level);
  } else {
    return safe.translateStackIOS(stack, level);
  }
};

var normalizeIndent = function(lines, pos) {
  pos = pos || 0;
  var label = lines[pos].match(/^[^\s]* /);
  if (label) {
    var indent = label[0].replace(/./g, ' ');
    for (var i = pos + 1, ii = lines.length; i < ii; i++) {
      lines[i] = lines[i].replace(/^\t/, indent);
    }
  }
  return lines;
};

safe.translateError = function(err, intro, level) {
  var name = err.name;
  var message = err.message || err.toString();
  var stack = err.stack;
  var result = [intro || 'JavaScript Error:'];
  if (message && (!stack || stack.indexOf(message) === -1)) {
    if (name && message.indexOf(name + ':') === -1) {
      message = name + ': ' + message;
    }
    result.push(message);
  }
  if (stack) {
    Array.prototype.push.apply(result, safe.translateStack(stack, level));
  }
  return normalizeIndent(result, 1).join('\n');
};

/* Dumps error messages to the console. */
safe.dumpError = function(err, intro, level) {
  if (typeof err === 'object') {
    console.log(safe.translateError(err, intro, level));
  } else {
    console.log('Error: dumpError argument is not an object');
  }
};

/* Logs runtime warnings to the console. */
safe.warn = function(message, level, name) {
  var err = new Error(message);
  err.name = name || 'Warning';
  safe.dumpError(err, 'Warning:', 1);
};

/* Takes a function and return a new function with a call to it wrapped in a try/catch statement */
safe.protect = function(fn) {
  return fn ? function() {
    try {
      fn.apply(this, arguments);
    } catch (err) {
      safe.dumpError(err);
    }
  } : undefined;
};

/* Wrap event handlers added by Pebble.addEventListener */
var pblAddEventListener = Pebble.addEventListener;
Pebble.addEventListener = function(eventName, eventCallback) {
  pblAddEventListener.call(this, eventName, safe.protect(eventCallback));
};

var pblSendMessage = Pebble.sendAppMessage;
Pebble.sendAppMessage = function(message, success, failure) {
  return pblSendMessage.call(this, message, safe.protect(success), safe.protect(failure));
};

/* Wrap setTimeout and setInterval */
var originalSetTimeout = setTimeout;
window.setTimeout = function(callback, delay) {
  if (safe.warnSetTimeoutNotFunction !== false && typeof callback !== 'function') {
    safe.warn('setTimeout was called with a `' + (typeof callback) + '` type. ' +
              'Did you mean to pass a function?');
    safe.warnSetTimeoutNotFunction = false;
  }
  return originalSetTimeout(safe.protect(callback), delay);
};

var originalSetInterval = setInterval;
window.setInterval = function(callback, delay) {
  if (safe.warnSetIntervalNotFunction !== false && typeof callback !== 'function') {
    safe.warn('setInterval was called with a `' + (typeof callback) + '` type. ' +
              'Did you mean to pass a function?');
    safe.warnSetIntervalNotFunction = false;
  }
  return originalSetInterval(safe.protect(callback), delay);
};

/* Wrap the geolocation API Callbacks */
var watchPosition = navigator.geolocation.watchPosition;
navigator.geolocation.watchPosition = function(success, error, options) {
  return watchPosition.call(this, safe.protect(success), safe.protect(error), options);
};

var getCurrentPosition = navigator.geolocation.getCurrentPosition;
navigator.geolocation.getCurrentPosition = function(success, error, options) {
  return getCurrentPosition.call(this, safe.protect(success), safe.protect(error), options);
};

var ajax;

/* Try to load the ajax library if available and silently fail if it is not found. */
try {
  ajax = require('ajax');
} catch (err) {}

/* Wrap the success and failure callback of the ajax library */
if (ajax) {
  ajax.onHandler = function(eventName, callback) {
    return safe.protect(callback);
  };
}

module.exports = safe;
});
__loader.define('src/js/app.js', 397, function(exports, module, require) {
//"use strict";

// XMLHttpRequest helper
var xhrRequest = function (url, type, callback) {
	var xhr = new XMLHttpRequest();
	xhr.onload = function () {
		callback(this.responseText);
	};
	xhr.open(type, url);
	xhr.send();
};

function getPokemon(latitude, longitude) {

	// live PokeVision data, hard-coded to Ann Arbor for now
	var scanUrl = 'https://pokevision.com/map/scan/' + latitude + '/' + longitude;
	var dataUrl = 'https://pokevision.com/map/data/' + latitude + '/' + longitude;

	// static (stable!) example of PokeVision data
	//var scanUrl = 'https://mathewreiss.github.io/PoGO/data.json';
	//var dataUrl = 'https://mathewreiss.github.io/PoGO/data.json';

	// TODO: is this OK?
	xhrRequest(scanUrl, 'GET', 
		function(scanResponseText) {

			// TODO: need to check for "Maintenance..." page - may not even be JSON!

			var scanJson = JSON.parse(scanResponseText);
			console.log(scanResponseText); // JSON.stringify() not necessary!

			// TODO: check scanResponseText success (although...does throttling error matter
			// since we can still view pokes from last scan...?

			xhrRequest(dataUrl, 'GET', 
				function(dataResponseText) {
					var json = JSON.parse(dataResponseText);
					console.log(dataResponseText); // JSON.stringify() not necessary!

					// TODO: status check!
					console.log('status is "' + json.status + '"');

					// TODO: much better error checking???
					if (json.pokemon.length > 0) {

						var allNearbyPokemon = [];

						var i;
						for (i = 0; i < json.pokemon.length - 1; i++) {

							// TODO: should still actually verify vs. using blindly!
							console.log('pokemon[' + i + '].pokemonId is "' + json.pokemon[i].pokemonId + '"');
							// PokeVision is string for some reason
							var pokemonId = Number(json.pokemon[i].pokemonId);
							console.log('pokemonId is "' + pokemonId + '"');

							var pokemonExpirationTime = json.pokemon[i].expiration_time;
							console.log('pokemonExpirationTime is "' + pokemonExpirationTime + '"');

							var pokemonLatitude = json.pokemon[i].latitude;
							console.log('pokemonLatitude is "' + pokemonLatitude + '"');
							var pokemonLongitude = json.pokemon[i].longitude;
							console.log('pokemonLongitude is "' + pokemonLongitude + '"');

							var pokemonDistance = getDistance(latitude, longitude, pokemonLatitude, pokemonLongitude);		

							// fails on iOS!
							// per @katharine:
							// > PebbleKit JS Android is not to spec.
							//allNearbyPokemon.push({i, pokemonId, pokemonExpirationTime, pokemonDistance});

							var pokemonData = {
								"i": i, 
								"pokemonId": pokemonId, 
								"pokemonExpirationTime": pokemonExpirationTime, 
								"pokemonDistance": pokemonDistance
							};
							allNearbyPokemon.push(pokemonData);

						}

						console.log("allNearbyPokemon: " + JSON.stringify(allNearbyPokemon));

						// sort by distance
						allNearbyPokemon.sort(function(a, b) {
						    return a.pokemonDistance - b.pokemonDistance;
						});

						// Assemble dictionary using our keys
						var dictionary = {};

						// take closest 9 (or fewer if not available; sentinel indicated by pokemonId == 0)
						var j;
						for (j = 0; j < 9; j++) {

							if (j < json.pokemon.length - 1) {
								dictionary["Pokemon" + (j + 1) + "Id"] = allNearbyPokemon[j].pokemonId;
								dictionary["Pokemon" + (j + 1) + "ExpirationTime"] = allNearbyPokemon[j].pokemonExpirationTime;
								dictionary["Pokemon" + (j + 1) + "Distance"] = allNearbyPokemon[j].pokemonDistance;								
							} else {
								dictionary["Pokemon" + (j + 1) + "Id"] = 0;
								dictionary["Pokemon" + (j + 1) + "ExpirationTime"] = 0;
								dictionary["Pokemon" + (j + 1) + "Distance"] = 0;								
								break;
							}

						}

						console.log("dictionary: " + JSON.stringify(dictionary));

						// Send to Pebble
						Pebble.sendAppMessage(dictionary,
							function(e) {
								console.log("AppMessage sent to Pebble successfully!");
							},
							function(e) {
								console.log("Error sending AppMessage to Pebble!");
							}
						);
					} else {
						// no pokemon found!
						Pebble.showSimpleNotificationOnPebble("No Pokemon found!", "(" + latitude + ", " + longitude + ")");
					}


				}
			);

		}
	);

}


function geolocationSuccess(pos) {
  console.log('lat= ' + pos.coords.latitude + ' lon= ' + pos.coords.longitude);

  getPokemon(pos.coords.latitude, pos.coords.longitude);
}

function geolocationError(err) {
  console.log('location error (' + err.code + '): ' + err.message);

  // TODO: alert user via watchapp instead
  Pebble.showSimpleNotificationOnPebble("location error", err.message);
}

function updateLocation() {
	var options = {
	  enableHighAccuracy: true,
	  maximumAge: 10000,
	  timeout: 10000
	};

	navigator.geolocation.getCurrentPosition(geolocationSuccess, geolocationError, options);
}

// Listen for when the watchface is opened
Pebble.addEventListener('ready', 
	function(e) {
		console.log('PebbleKit JS ready!');

		updateLocation();
	}
);

// Listen for when an AppMessage is received
Pebble.addEventListener('appmessage',
	function(e) {
		console.log('AppMessage received!');

		updateLocation();
	}                     
);


// based on @mathew's process_distance()
function getDistance(myLatitude, myLongitude, pkmnLatitude, pkmnLongitude) {
	var distance;

	var lat1 = myLatitude, lon1 = myLongitude;
	var lat2 = pkmnLatitude, lon2 = pkmnLongitude;
	
	var dLat = toRadians(lat2-lat1);
	var dLon = toRadians(lon2-lon1);
	
	lat1 = toRadians(lat1);
	lat2 = toRadians(lat2);
	
	var y = Math.sin(dLon) * Math.cos(lat2);
	var x = Math.cos(lat1) * Math.sin(lat2) - Math.sin(lat1) * Math.cos(lat2) * Math.cos(dLon);
		
	var a = Math.sin(dLat/2) * Math.sin(dLat/2) + Math.cos(lat1) * Math.cos(lat2) * Math.sin(dLon/2) * Math.sin(dLon/2);
	var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
	
	//distance = convert_units(c*3959);
	// numeric (meters) for now since actual data doesn't appear to reach kms
	distance = (c*3959) * 1.60934 * 1000;
	
	console.log("Start Lat: " + lat1);
	console.log("Start Lon: " + lon1);
	console.log("End Lat: " + lat2);
	console.log("End Lon: " + lon2);
	console.log("Distance: " + distance);

	return distance;
}

function getBearing(myLatitude, myLongitude, pkmnLatitude, pkmnLongitude) {
	var bearing;

	var lat1 = myLatitude, lon1 = myLongitude;
	var lat2 = pkmnLatitude, lon2 = pkmnLongitude;
	
	var dLat = toRadians(lat2-lat1);
	var dLon = toRadians(lon2-lon1);
	
	lat1 = toRadians(lat1);
	lat2 = toRadians(lat2);
	
	var y = Math.sin(dLon) * Math.cos(lat2);
	var x = Math.cos(lat1) * Math.sin(lat2) - Math.sin(lat1) * Math.cos(lat2) * Math.cos(dLon);
	
	bearing = toDegrees(Math.atan2(y,x));
	if(bearing < 0) bearing = 360-Math.abs(bearing);
	
	console.log("Start Lat: " + lat1);
	console.log("Start Lon: " + lon1);
	console.log("End Lat: " + lat2);
	console.log("End Lon: " + lon2);
	console.log("Bearing: " + bearing);

	return bearing;
}

// TODO: move to own .js or maybe @mathew already has a lib?

function toRadians(degrees) {
	return degrees * Math.PI / 180;
}

function toDegrees(radians) {
	return radians * 180 / Math.PI;
}

function convert_units(old_distance) {
	var new_distance;
	
	old_distance *= 1.60934; //Convert miles to km
	
	if(old_distance < 0.1){
		new_distance = old_distance * 1000.0;
		
		if(new_distance < accuracy){
			new_distance = "< " + (accuracy/10).toFixed(0)*10 + " m";
		}
		else{
			new_distance += "";
			new_distance = new_distance.substring(0, new_distance.indexOf('.') + 2);
			new_distance += " m";
		}
	}
	else{
		new_distance = old_distance + "";
		new_distance = new_distance.substring(0, new_distance.indexOf('.') + 3);
		new_distance += " km";
	}
	//}
	
	if(new_distance.charAt(0) === '.') new_distance = "0" + new_distance;
	
	return new_distance;
}
});
__loader.define('build/js/message_keys.json', 672, function(exports, module, require) {
module.exports = {
    "Pokemon1Distance": 2,
    "Pokemon1ExpirationTime": 1,
    "Pokemon1Id": 0,
    "Pokemon2Distance": 5,
    "Pokemon2ExpirationTime": 4,
    "Pokemon2Id": 3,
    "Pokemon3Distance": 8,
    "Pokemon3ExpirationTime": 7,
    "Pokemon3Id": 6,
    "Pokemon4Distance": 11,
    "Pokemon4ExpirationTime": 10,
    "Pokemon4Id": 9,
    "Pokemon5Distance": 14,
    "Pokemon5ExpirationTime": 13,
    "Pokemon5Id": 12,
    "Pokemon6Distance": 17,
    "Pokemon6ExpirationTime": 16,
    "Pokemon6Id": 15,
    "Pokemon7Distance": 20,
    "Pokemon7ExpirationTime": 19,
    "Pokemon7Id": 18,
    "Pokemon8Distance": 23,
    "Pokemon8ExpirationTime": 22,
    "Pokemon8Id": 21,
    "Pokemon9Distance": 26,
    "Pokemon9ExpirationTime": 25,
    "Pokemon9Id": 24,
    "RequestType": 27
};
});
(function() {
  var safe = __loader.require('safe');
  safe.protect(function() {
    __loader.require('/src/js/app');
  })();
})();